@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2015-2018 by the Free Software Foundation, Inc.
+# Copyright (C) 2015-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -17,12 +17,16 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
 
+import logging
+from urllib.error import HTTPError
+
+from mailmanclient import MailmanConnectionError
+
 from postorius import utils
 from postorius.models import MailmanApiError
-from mailmanclient import MailmanConnectionError
-import logging
 
-logger = logging.getLogger(__name__)
+
+logger = logging.getLogger('postorius')
 
 
 __all__ = [
@@ -42,3 +46,6 @@ class PostoriusMiddleware(object):
         if isinstance(exception, (MailmanApiError, MailmanConnectionError)):
             logger.exception('Mailman REST API not available')
             return utils.render_api_error(request)
+        elif isinstance(exception, HTTPError):
+            logger.exception('Un-handled exception: %s', str(exception))
+            return utils.render_client_error(request, exception)
