@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 1998-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -16,29 +16,53 @@
 # You should have received a copy of the GNU General Public License along with
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import (
-    absolute_import, division, print_function, unicode_literals)
-
 import logging
+from enum import Enum
+from urllib.error import HTTPError
 from urllib.parse import urljoin
 
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
 from django.contrib.auth.models import User
+from django.core.exceptions import ImproperlyConfigured
 from django.db import models
-from django.db.models.signals import post_save, post_delete
+from django.db.models.signals import post_delete, post_save
 from django.dispatch import receiver
 from django.http import Http404
 from django.urls import reverse
-from django.utils.six.moves.urllib.error import HTTPError
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
+
 from mailmanclient import MailmanConnectionError
 
-from postorius.utils import get_mailman_client, LANGUAGES
 from postorius.template_list import TEMPLATES_LIST
+from postorius.utils import LANGUAGES, get_mailman_client
+
 
 logger = logging.getLogger(__name__)
 
+_email_template_help_text = _(
+    'Note: Do not add any secret content in templates as they are '
+    'publicly accessible.\n'
+    'You can use these variables in the templates. \n'
+    '$hyperkitty_url: Permalink to archived message in Hyperkitty\n'
+    '$listname: Name of the Mailing List e.g. ant@example.com \n'
+    '$list_id: The List-ID header e.g. ant.example.com \n'
+    '$display_name: Display name of the mailing list e.g. Ant \n'
+    '$short_listname: Local part of the listname e.g. ant \n'
+    '$domain: The domain part of the listname e.g. example.com \n'
+    '$info: The mailing list\'s longer descriptive text \n'
+    '$request_email: The email address for -request address \n'
+    '$owner_email: The email address for -owner address \n'
+    '$site_email: The email address to reach the owners of the site \n'
+    '$language: The two letter language code for list\'s preferred language e.g. fr, en, de \n'  # noqa: E501
+    )
+
+
+class SubscriptionMode(Enum):
+    """Valid values for Member.subscription_mode"""
+
+    as_address = 1
+    as_user = 2
+
 
 @receiver(post_save, sender=User)
 def create_mailman_user(sender, **kwargs):
@@ -226,32 +250,26 @@ class EmailTemplate(models.Model):
     Footers on emails for decorations are also repsented as templates.
     """
 
+    # Ease differentiating the various Mailman templates by providing the
+    # template file's name (key) prepended in square brackets to the
+    # template's purpose (value).
+    _templates_list_choices = [
+        (t[0], "[{key}] - {value}".format(key=t[0], value=t[1]))
+        for t in TEMPLATES_LIST
+    ]
+
     name = models.CharField(
-        max_length=100, choices=TEMPLATES_LIST,
+        max_length=100, choices=_templates_list_choices,
         help_text=_('Choose the template you want to customize.'))
     data = models.TextField(
-        help_text=_(
-            'Note: Do not add any secret content in templates as they are '
-            'publicly accessible.\n'
-            'You can use these variables in the templates. \n'
-            '$hyperkitty_url: Permalink to archived message in Hyperkitty\n'
-            '$listname: Name of the Mailing List e.g. ant@example.com \n'
-            '$list_id: The List-ID header e.g. ant.example.com \n'
-            '$display_name: Display name of the mailing list e.g. Ant \n'
-            '$short_listname: Local part of the listname e.g. ant \n'
-            '$domain: The domain part of the listname e.g. example.com \n'
-            '$info: The mailing list\'s longer descriptive text \n'
-            '$request_email: The email address for -request address \n'
-            '$owner_email: The email address for -owner address \n'
-            '$site_email: The email address to reach the owners of the site \n'
-            '$language: The two letter language code for list\'s preferred language e.g. fr, en, de \n'  # noqa: E501
-        )
+        help_text=_email_template_help_text,
+        blank=True,
     )
     language = models.CharField(
         max_length=5, choices=LANGUAGES,
         help_text=_('Language for the template, this should be the list\'s preferred language.'),     # noqa: E501
         blank=True)
-    craeted_at = models.DateTimeField(auto_now_add=True)
+    created_at = models.DateTimeField(auto_now_add=True)
     modified_at = models.DateTimeField(auto_now=True)
     context = models.CharField(max_length=50, choices=TEMPLATE_CONTEXT_CHOICES)
     identifier = models.CharField(blank=True, max_length=100)
@@ -272,7 +290,8 @@ class EmailTemplate(models.Model):
         """API url is the remote url that Core can use to fetch templates"""
         base_url = getattr(settings, 'POSTORIUS_TEMPLATE_BASE_URL', None)
         if not base_url:
-            raise ImproperlyConfigured
+            raise ImproperlyConfigured(
+                'Setting "POSTORIUS_TEMPLATE_BASE_URL" is not configured.')
         resource_url = reverse(
             'rest_template',
             kwargs=dict(context=self.context,
