@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2016-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -17,14 +17,14 @@
 
 """Tests for list settings"""
 
-from allauth.account.models import EmailAddress
 from django.contrib.auth.models import User
 from django.urls import reverse
-from django.utils import six
 
-from postorius.views.list import SETTINGS_FORMS
+from allauth.account.models import EmailAddress
+
 from postorius.models import List
 from postorius.tests.utils import ViewTestCase
+from postorius.views.list import SETTINGS_FORMS
 
 
 class ListSettingsTest(ViewTestCase):
@@ -50,6 +50,9 @@ class ListSettingsTest(ViewTestCase):
         self.foo_list.add_owner('owner@example.com')
         self.foo_list.add_moderator('moderator@example.com')
 
+    def tearDown(self):
+        self.domain.delete()
+
     def test_page_not_accessible_if_not_logged_in(self):
         for section_name in SETTINGS_FORMS:
             url = reverse('list_settings', args=('foo.example.com',
@@ -154,6 +157,9 @@ class ListSettingsTest(ViewTestCase):
             'subject_prefix': '',
             'description': '',
             'advertised': 'True',
+            'preferred_language': 'en',
+            'member_roster_visibility': 'public',
+            'newsgroup_moderation': 'none',
             })
         self.assertRedirects(response, url)
         self.assertHasSuccessMessage(response)
@@ -174,6 +180,7 @@ class ListSettingsTest(ViewTestCase):
              'autorespond_postings': 'none',
              'autorespond_requests': 'none',
              'send_welcome_message': True,
+             'send_goodbye_message': True,
              'autoresponse_grace_period': '20d'})
         self.assertRedirects(response, url)
         self.assertHasSuccessMessage(response)
@@ -270,7 +277,7 @@ class ListSettingsTest(ViewTestCase):
                       args=('foo.example.com', 'message_acceptance'))
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
-        for field, value in six.iteritems(initial_values):
+        for field, value in initial_values.items():
             self.assertEqual(
                 self.foo_list.settings[field], value,
                 'Field: {}'.format(field))
@@ -291,7 +298,104 @@ class ListSettingsTest(ViewTestCase):
         self.assertHasSuccessMessage(response)
         # Get a new list object to avoid caching
         m_list = List.objects.get(fqdn_listname='foo.example.com')
-        for field, value in six.iteritems(updated_values):
+        for field, value in updated_values.items():
             self.assertEqual(
                 m_list.settings[field], value,
                 'Field: {}'.format(field))
+
+    def test_mass_subscribe_options(self):
+        self.client.login(username='testsu', password='testpass')
+        mlist = List.objects.get(fqdn_listname='foo.example.com')
+        updated_values = {
+            'emails': 'john3@example.com',
+            'pre_verified': True,
+            'pre_confirmed': True,
+            'pre_approved': True,
+        }
+        url = reverse('mass_subscribe',
+                      args=('foo.example.com',))
+        response = self.client.post(url, updated_values)
+
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(mlist.members), 1)
+
+        # Now, let's see if we can moderate.
+        self.foo_list.settings['subscription_policy'] = 'moderate'
+        self.foo_list.settings.save()
+
+        updated_values = {
+            'emails': 'john@example.com',
+            'pre_confirmed': True,
+            'pre_approved': True,
+            'pre_verified': True
+        }
+
+        self.assertEqual(len(mlist.members), 1)
+        response = self.client.post(url, updated_values)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(mlist.members), 2)
+        # With pre_confirmed = False, there should be a pending subscription
+        # event.
+        updated_values = {
+            'emails': 'john2@example.com',
+            'pre_confirmed': True,
+            'pre_verified': True
+        }
+        response = self.client.post(url, updated_values)
+        self.assertEqual(response.status_code, 200)
+        # Now, there should be a pending request to confirm.
+        self.assertEqual(len(mlist.members), 2)
+        self.assertEqual(len(mlist.requests), 1)
+        self.assertEqual(mlist.requests[0].get('token_owner'), 'moderator')
+
+        # Test mass subscribe without all options.
+        self.foo_list.settings['subscription_policy'] = 'confirm_then_moderate'
+        self.foo_list.settings.save()
+
+        updated_values = {
+            'emails': 'john4@example.com',
+            'pre_confirmed': False,
+            'pre_verified': True
+        }
+
+        response = self.client.post(url, updated_values)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(mlist.requests), 2)
+        self.assertEqual(mlist.requests[1].get('token_owner'), 'subscriber')
+
+        # Test mass subscribe an invitation.  Subscription policy doesn't
+        # matter, so just leave it as it was.
+        updated_values = {
+            'emails': 'john5@example.com',
+            'invitation': True
+        }
+        response = self.client.post(url, updated_values)
+        self.assertEqual(response.status_code, 200)
+        # And now there are 3 requests.
+        self.assertEqual(len(mlist.requests), 3)
+        self.assertEqual(mlist.requests[2].get('token_owner'), 'subscriber')
+
+    def test_set_list_of_strings_field_empty(self):
+        self.client.login(username='testsu', password='testpass')
+        m_list = List.objects.get(fqdn_listname='foo.example.com')
+        settings = m_list.settings
+        settings['accept_these_nonmembers'] = ['^bar*@example.[com|org]']
+        settings.save()
+
+        url = reverse('list_settings',
+                      args=('foo.example.com', 'message_acceptance'))
+        updated_values = {
+            'accept_these_nonmembers': [],
+            'require_explicit_destination': True,
+            'administrivia': True,
+            'default_member_action': 'defer',
+            'default_nonmember_action': 'hold',
+            'max_message_size': 40,
+            'max_num_recipients': 10,
+        }
+        response = self.client.post(url, updated_values)
+        self.assertEqual(response.status_code, 302)
+        self.assertHasSuccessMessage(response)
+        # Get a new list object to avoid caching
+        settings._reset_cache()
+        self.assertEqual(settings.get('accept_these_nonmembers'), [])
