@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -15,13 +15,13 @@
 # You should have received a copy of the GNU General Public License along with
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
+from unittest.mock import MagicMock, patch
 from urllib.error import HTTPError
 
 from django.contrib.auth.models import User
 from django.test import override_settings
 
 from allauth.account.models import EmailAddress
-from mock import MagicMock, patch
 
 from postorius.models import (
     MailmanApiError, MailmanListManager, MailmanUserManager)
