@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -16,19 +16,20 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from allauth.account.models import EmailAddress
+from unittest.mock import patch
+
 from django.contrib.auth.models import AnonymousUser, User
 from django.core.exceptions import PermissionDenied
-from django.test.client import RequestFactory
 from django.test import TestCase
-from mock import patch
+from django.test.client import RequestFactory
 
-from postorius.auth.decorators import (list_owner_required,
-                                       list_moderator_required,
-                                       superuser_required)
-from postorius.tests.utils import create_mock_list
+from allauth.account.models import EmailAddress
 from mailmanclient import Client
 
+from postorius.auth.decorators import (
+    list_moderator_required, list_owner_required, superuser_required)
+from postorius.tests.utils import create_mock_list
+
 
 @list_owner_required
 def dummy_function(request, list_id):
