@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -17,21 +17,21 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from django.conf.urls import url, include
+from django.conf.urls import include, url
 
-from postorius.views import user as user_views
 from postorius.views import domain as domain_views
 from postorius.views import list as list_views
 from postorius.views import rest as rest_views
 from postorius.views import system as system_views
 from postorius.views import template as template_views
+from postorius.views import user as user_views
 
 
 list_patterns = [
     url(r'^csv_view/$', list_views.csv_view, name='csv_view'),
     url(r'^members/options/(?P<email>.+)$', list_views.list_member_options,
         name='list_member_options'),
-    url(r'^members/(?P<role>\w+)/$', list_views.list_members_view,
+    url(r'^members/(?P<role>\w+)/$', list_views.ListMembersViews.as_view(),
         name='list_members'),
     url(r'^$', list_views.ListSummaryView.as_view(),
         name='list_summary'),
@@ -46,6 +46,8 @@ list_patterns = [
         name='list_unsubscribe'),
     url(r'^subscription_requests$', list_views.list_subscription_requests,
         name='list_subscription_requests'),
+    url(r'^pending_confirmation$', list_views.list_pending_confirmations,
+        name='list_pending_confirmation'),
     url(r'^handle_subscription_request/(?P<request_id>[^/]+)/'
         '(?P<action>[accept|reject|discard|defer]+)$',
         list_views.handle_subscription_request,
@@ -68,6 +70,8 @@ list_patterns = [
         name='list_settings'),
     url(r'^unsubscribe_all$', list_views.remove_all_subscribers,
         name='unsubscribe_all'),
+    url(r'^confirm/$', list_views.confirm_token,
+        name='confirm_token'),
     url(r'^templates$',
         template_views.ListTemplateIndexView.as_view(),
         name='list_template_list'),
@@ -83,7 +87,7 @@ list_patterns = [
 ]
 
 urlpatterns = [
-    url(r'^$', list_views.list_index),
+    url(r'^$', list_views.list_index),                   # noqa: W605 (bogus)
     url(r'^accounts/subscriptions/$', user_views.user_subscriptions,
         name='ps_user_profile'),
     url(r'^accounts/per-address-preferences/$',
@@ -106,6 +110,11 @@ urlpatterns = [
         name='domain_edit'),
     url(r'^domains/(?P<domain>[^/]+)/delete$', domain_views.domain_delete,
         name='domain_delete'),
+    url(r'^domains/(?P<domain>[^/]+)/owners$', domain_views.domain_owners,
+        name='domain_owners'),
+    url(r'^domains/(?P<domain>[^/]+)/owners/(?P<user_id>.+)/remove$',
+        domain_views.remove_owners,
+        name='remove_domain_owner'),
     # Ideally, these paths should be accessible by domain_owners, however,
     # we don't have good ways to check that, so for now, this views are
     # protected by superuser privileges.
@@ -127,13 +136,18 @@ urlpatterns = [
     url(r'^lists/new/$', list_views.list_new, name='list_new'),
     url(r'^lists/(?P<list_id>[^/]+)/', include(list_patterns)),
 
+    # /system/
     url(r'^system/$', system_views.system_information,
         name='system_information'),
 
+    # /bans/
+    url(r'^bans/$', system_views.bans, name='global_bans'),
+
+    # /api/
     url(r'^api/list/(?P<list_id>[^/]+)/held_message/(?P<held_id>\d+)/$',
         rest_views.get_held_message, name='rest_held_message'),
     url(r'^api/list/(?P<list_id>[^/]+)/held_message/(?P<held_id>\d+)/'
-        'attachment/(?P<attachment_id>\d+)/$',
+        r'attachment/(?P<attachment_id>\d+)/$',
         rest_views.get_attachment_for_held_message,
         name='rest_attachment_for_held_message'),
     # URL configuration for templates.
