@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 1998-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -20,40 +20,55 @@
 import csv
 import email.utils
 import logging
+import sys
+from urllib.error import HTTPError
 
-from allauth.account.models import EmailAddress
-from django.http import HttpResponse, HttpResponseNotAllowed, Http404
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth.decorators import login_required
+from django.core.exceptions import ValidationError
 from django.core.validators import validate_email
 from django.forms import formset_factory
-from django.shortcuts import render, redirect
-from django.core.exceptions import ValidationError
+from django.http import Http404, HttpResponse
+from django.shortcuts import redirect, render
 from django.urls import reverse
 from django.utils.decorators import method_decorator
 from django.utils.translation import gettext as _
-from django_mailman3.lib.mailman import get_mailman_client
-from django_mailman3.lib.paginator import paginate, MailmanPaginator
-from django.utils.six.moves.urllib.error import HTTPError
+from django.views.decorators.http import require_http_methods
+
+from allauth.account.models import EmailAddress
+from django_mailman3.lib.mailman import get_mailman_client, get_mailman_user
+from django_mailman3.lib.paginator import MailmanPaginator, paginate
+from django_mailman3.models import MailDomain
+from django_mailman3.signals import (
+    mailinglist_created, mailinglist_deleted, mailinglist_modified)
 
-from postorius.forms import (
-    ListNew, MemberForm, ListSubscribe, MultipleChoiceForm, UserPreferences,
-    ListSubscriptionPolicyForm, ArchiveSettingsForm, MessageAcceptanceForm,
-    DigestSettingsForm, AlterMessagesForm, ListAutomaticResponsesForm,
-    ListIdentityForm, ListMassSubscription, ListMassRemoval, ListAddBanForm,
-    ListHeaderMatchForm, ListHeaderMatchFormset, MemberModeration,
-    DMARCMitigationsForm, ListAnonymousSubscribe)
-from postorius.models import Domain, List, Mailman404Error, Style
 from postorius.auth.decorators import (
-    list_owner_required, list_moderator_required, superuser_required)
+    list_moderator_required, list_owner_required, superuser_required)
 from postorius.auth.mixins import ListOwnerMixin
-from postorius.views.generic import MailingListView
+from postorius.forms import (
+    AlterMessagesForm, ArchiveSettingsForm, BounceProcessingForm,
+    DigestSettingsForm, DMARCMitigationsForm, ListAnonymousSubscribe,
+    ListAutomaticResponsesForm, ListHeaderMatchForm, ListHeaderMatchFormset,
+    ListIdentityForm, ListMassRemoval, ListMassSubscription, ListNew,
+    ListSubscribe, MemberForm, MemberModeration, MemberPolicyForm,
+    MessageAcceptanceForm, MultipleChoiceForm, UserPreferences)
+from postorius.forms.list_forms import ACTION_CHOICES, TokenConfirmForm
+from postorius.models import (
+    Domain, List, Mailman404Error, Style, SubscriptionMode)
+from postorius.utils import get_member_or_nonmember, set_preferred
+from postorius.views.generic import MailingListView, bans_view
 
 
 logger = logging.getLogger(__name__)
 
 
+class TokenOwner:
+    """Who 'owns' the token returned from the registrar?"""
+    subscriber = 'subscriber'
+    moderator = 'moderator'
+
+
 class ListMembersViews(ListOwnerMixin, MailingListView):
 
     # List of allowed roles for the memberships. The string value matches the
@@ -138,13 +153,12 @@ class ListMembersViews(ListOwnerMixin, MailingListView):
                     display_name=member_form.cleaned_data['display_name'])
                 messages.success(
                     request,
-                    _('%(email)s has been added with the role %(role)s'.format(
-                        email=member_form.cleaned_data['email'],
-                        role=role)))
+                    _('{email} has been added with the role {role}'.format(
+                        email=member_form.cleaned_data['email'], role=role)))
             except HTTPError as e:
-                messages.error(request, e.msg.decode())
+                messages.error(request, e.msg)
         else:
-            messages.error(request, member_form.errorsg)
+            messages.error(request, member_form.errors)
         return redirect('list_members', self.mailing_list.list_id, role)
 
     def post(self, request, list_id, role=None):
@@ -165,7 +179,7 @@ class ListMembersViews(ListOwnerMixin, MailingListView):
         if role not in self.allowed_roles:
             return redirect('list_members', list_id, 'member')
 
-        if role in ('member'):
+        if role in ('member', ):
             return self._member_post(request, role)
         else:
             return self._non_member_post(request, role)
@@ -176,16 +190,24 @@ class ListMembersViews(ListOwnerMixin, MailingListView):
 def list_member_options(request, list_id, email):
     template_name = 'postorius/lists/memberoptions.html'
     mm_list = List.objects.get_or_404(fqdn_listname=list_id)
+    # If the role is specified explicitly, use that, otherwise the value is
+    # None and is equivalent to the value not being specified and the lookup
+    # happens using only email. This can cause issues when the email is
+    # subscribed as both a Non-Member and Owner/Moderator returning the wrong
+    # Member object.
+    role = request.GET.get('role')
     try:
-        mm_member = mm_list.find_members(address=email)[0]
+        mm_member = mm_list.find_members(address=email, role=role)[0]
         member_prefs = mm_member.preferences
     except (ValueError, IndexError):
         raise Http404(_('Member does not exist'))
     except Mailman404Error:
         return render(request, template_name, {'nolists': 'true'})
+
     initial_moderation = dict([
         (key, getattr(mm_member, key)) for key in MemberModeration.base_fields
         ])
+
     if request.method == 'POST':
         if request.POST.get("formname") == 'preferences':
             preferences_form = UserPreferences(
@@ -194,7 +216,7 @@ def list_member_options(request, list_id, email):
                 try:
                     preferences_form.save()
                 except HTTPError as e:
-                    messages.error(request, e.msg.decode())
+                    messages.error(request, e.msg)
                 else:
                     messages.success(request, _("The member's preferences"
                                                 " have been updated."))
@@ -215,7 +237,7 @@ def list_member_options(request, list_id, email):
                 try:
                     mm_member.save()
                 except HTTPError as e:
-                    messages.error(request, e.msg.decode())
+                    messages.error(request, e.msg)
                 else:
                     messages.success(request, _("The member's moderation "
                                                 "settings have been updated."))
@@ -239,6 +261,7 @@ class ListSummaryView(MailingListView):
         data = {'list': self.mailing_list,
                 'user_subscribed': False,
                 'subscribed_address': None,
+                'subscribed_preferred': False,
                 'public_archive': False,
                 'hyperkitty_enabled': False}
         if self.mailing_list.settings['archive_policy'] == 'public':
@@ -262,14 +285,25 @@ class ListSummaryView(MailingListView):
                     data['user_request_pending'] = True
                     break
                 try:
-                    self.mailing_list.get_member(address)
+                    member = self.mailing_list.get_member(address)
                 except ValueError:
                     pass
                 else:
                     data['user_subscribed'] = True
                     data['subscribed_address'] = address
+                    data['subscribed_preferred'] = bool(
+                        member.subscription_mode ==
+                        SubscriptionMode.as_user.name)
                     break  # no need to test more addresses
-            data['subscribe_form'] = ListSubscribe(user_emails)
+            mm_user = get_mailman_user(request.user)
+            primary_email = None
+            if mm_user.preferred_address is None:
+                primary_email = set_preferred(request.user, mm_user)
+            else:
+                primary_email = mm_user.preferred_address.email
+            data['subscribe_form'] = ListSubscribe(
+                user_emails, user_id=mm_user.user_id,
+                primary_email=primary_email)
         else:
             user_emails = None
             data['anonymous_subscription_form'] = ListAnonymousSubscribe()
@@ -280,18 +314,83 @@ class ChangeSubscriptionView(MailingListView):
     """Change mailing list subscription
     """
 
+    def _is_subscribed(self, member, subscriber):
+        """Check if the member is same as the subscriber."""
+        return (
+            (member.subscription_mode == SubscriptionMode.as_address.name and
+             member.email == subscriber) or
+            (member.subscription_mode == SubscriptionMode.as_user.name and
+             member.user.user_id == subscriber))
+
+    @staticmethod
+    def _is_email(subscriber):
+        """Is the subscriber an email address or uuid."""
+        return '@' in subscriber
+
+    def _change_subscription(self, member, subscriber):
+        """Switch subscriptions to a new email/user.
+
+        :param member: The currently subscriber Member.
+        :param subscriber: New email or user_id to switch subscription to.
+        """
+        # If the Membership is via preferred address and we are switching to an
+        # email or vice versa, we have to do the subscribe-unsubscribe dance.
+        # If the subscription is via an address, then we can simply PATCH the
+        # address in the Member resource.
+        if (member.subscription_mode == SubscriptionMode.as_address.name and
+                self._is_email(subscriber)):
+            member.address = subscriber
+            member.save()
+            return
+        # Keep a copy of the preferences so we can restore them after the
+        # Member switches addresses.
+        if member.preferences:
+            prefs = member.preferences
+        member.unsubscribe()
+        # Unless the subscriber is an an email and it is not-verified, we
+        # should get a Member resource here as response. We should never get to
+        # here with subscriber being an un-verified email.
+        new_member = self.mailing_list.subscribe(
+            subscriber,
+            # Since the email is already verified in Postorius.
+            pre_confirmed=True,
+            # Since this user was already a Member, simply switching Email
+            # addresses shouldn't require another approval.
+            pre_approved=True)
+        self._copy_preferences(new_member.preferences, prefs)
+
+    def _copy_preferences(self, member_pref, old_member_pref):
+        """Copy the preferences of the old member to new one.
+
+        :param member_pref: The new member's preference to copy preference to.
+        :param old_member_pref: The old_member's preferences to copy
+            preferences from.
+        """
+        # We can't simply switch the Preferences object, so we copy values from
+        # previous one to the new one.
+        for prop in old_member_pref._properties:
+            val = old_member_pref.get(prop)
+            if val:
+                member_pref[prop] = val
+        member_pref.save()
+
     @method_decorator(login_required)
     def post(self, request, list_id):
         try:
             user_emails = EmailAddress.objects.filter(
                 user=request.user, verified=True).order_by(
                 "email").values_list("email", flat=True)
-            form = ListSubscribe(user_emails, request.POST)
+            mm_user = get_mailman_user(request.user)
+            primary_email = None
+            if mm_user.preferred_address is not None:
+                primary_email = mm_user.preferred_address.email
+            form = ListSubscribe(
+                user_emails, mm_user.user_id, primary_email, request.POST)
             # Find the currently subscribed email
             old_email = None
             for address in user_emails:
                 try:
-                    self.mailing_list.get_member(address)
+                    member = self.mailing_list.get_member(address)
                 except ValueError:
                     pass
                 else:
@@ -299,30 +398,25 @@ class ChangeSubscriptionView(MailingListView):
                     break  # no need to test more addresses
             assert old_email is not None
             if form.is_valid():
-                email = form.cleaned_data['email']
-                if old_email == email:
+                subscriber = form.cleaned_data['subscriber']
+                if self._is_subscribed(member, subscriber):
                     messages.error(request, _('You are already subscribed'))
                 else:
-                    self.mailing_list.unsubscribe(old_email)
-                    # Since the action is done via the web UI, no email
-                    # confirmation is needed.
-                    response = self.mailing_list.subscribe(
-                        email, pre_confirmed=True)
-                    if (type(response) == dict and                # noqa: W504
-                            response.get('token_owner') == 'moderator'):
-                        messages.success(
-                            request, _('Your request to change the email for'
-                                       ' this subscription was submitted and'
-                                       ' is waiting for moderator approval.'))
-                    else:
-                        messages.success(request,
-                                         _('Subscription changed to %s') %
-                                         email)
+                    self._change_subscription(member, subscriber)
+                    # If the subscriber is user_id (i.e. not an email
+                    # address), Show 'Primary Address ()' in the success
+                    # message instead of weird user id.
+                    if not self._is_email(subscriber):
+                        subscriber = _(
+                            'Primary Address ({})').format(primary_email)
+                    messages.success(
+                        request,
+                        _('Subscription changed to %s').format(subscriber))
             else:
                 messages.error(request,
                                _('Something went wrong. Please try again.'))
         except HTTPError as e:
-            messages.error(request, e.msg.decode())
+            messages.error(request, e.msg)
         return redirect('list_summary', self.mailing_list.list_id)
 
 
@@ -341,13 +435,20 @@ class ListSubscribeView(MailingListView):
             user_emails = EmailAddress.objects.filter(
                 user=request.user, verified=True).order_by(
                 "email").values_list("email", flat=True)
-            form = ListSubscribe(user_emails, request.POST)
+            mm_user = get_mailman_user(request.user)
+            primary_email = None
+            if mm_user.preferred_address is not None:
+                primary_email = mm_user.preferred_address.email
+            form = ListSubscribe(
+                user_emails, mm_user.user_id, primary_email, request.POST)
             if form.is_valid():
-                email = request.POST.get('email')
+                subscriber = request.POST.get('subscriber')
+                display_name = request.POST.get('display_name')
                 response = self.mailing_list.subscribe(
-                    email, pre_verified=True, pre_confirmed=True)
-                if (type(response) == dict and                    # noqa: W504
-                        response.get('token_owner') == 'moderator'):
+                    subscriber, display_name,
+                    pre_verified=True, pre_confirmed=True)
+                if (type(response) == dict and                   # noqa: W504
+                        response.get('token_owner') == TokenOwner.moderator):
                     messages.success(
                         request, _('Your subscription request has been'
                                    ' submitted and is waiting for moderator'
@@ -360,7 +461,7 @@ class ListSubscribeView(MailingListView):
                 messages.error(request,
                                _('Something went wrong. Please try again.'))
         except HTTPError as e:
-            messages.error(request, e.msg.decode())
+            messages.error(request, e.msg)
         return redirect('list_summary', self.mailing_list.list_id)
 
 
@@ -380,15 +481,17 @@ class ListAnonymousSubscribeView(MailingListView):
             form = ListAnonymousSubscribe(request.POST)
             if form.is_valid():
                 email = form.cleaned_data.get('email')
-                self.mailing_list.subscribe(email, pre_verified=False,
-                                            pre_confirmed=False)
+                display_name = form.cleaned_data.get('display_name')
+                self.mailing_list.subscribe(
+                    email, display_name,
+                    pre_verified=False, pre_confirmed=False)
                 messages.success(request, _('Please check your inbox for '
                                             'further instructions'))
             else:
                 messages.error(request,
                                _('Something went wrong. Please try again.'))
         except HTTPError as e:
-            messages.error(request, e.msg.decode())
+            messages.error(request, e.msg)
         return redirect('list_summary', self.mailing_list.list_id)
 
 
@@ -420,11 +523,15 @@ def list_mass_subscribe(request, list_id):
                     # Parse the data to get the address and the display name
                     display_name, address = email.utils.parseaddr(data)
                     validate_email(address)
-                    mailing_list.subscribe(address=address,
-                                           display_name=display_name,
-                                           pre_verified=True,
-                                           pre_confirmed=True,
-                                           pre_approved=True)
+                    mailing_list.subscribe(
+                        address=address,
+                        display_name=display_name,
+                        pre_verified=form.cleaned_data['pre_verified'],
+                        pre_confirmed=form.cleaned_data['pre_confirmed'],
+                        pre_approved=form.cleaned_data['pre_approved'],
+                        invitation=form.cleaned_data['invitation'],
+                        send_welcome_message=form.cleaned_data[
+                            'send_welcome_message'])
                     messages.success(
                         request, _('The address %(address)s has been'
                                    ' subscribed to %(list)s.') %
@@ -458,20 +565,22 @@ class ListMassRemovalView(MailingListView):
         if not form.is_valid():
             messages.error(request, _('Please fill out the form correctly.'))
         else:
-            for address in form.cleaned_data['emails']:
+            for data in form.cleaned_data['emails']:
                 try:
+                    # Parse the data to get the address.
+                    address = email.utils.parseaddr(data)[1]
                     validate_email(address)
                     self.mailing_list.unsubscribe(address.lower())
                     messages.success(
                         request, _('The address %(address)s has been'
                                    ' unsubscribed from %(list)s.') %
-                        {'address': address,
+                        {'address': data,
                          'list': self.mailing_list.fqdn_listname})
                 except (HTTPError, ValueError) as e:
                     messages.error(request, e)
                 except ValidationError:
                     messages.error(request, _('The email address %s'
-                                              ' is not valid.') % address)
+                                              ' is not valid.') % data)
         return redirect('mass_removal', self.mailing_list.list_id)
 
 
@@ -513,26 +622,54 @@ def list_moderation(request, list_id, held_id=-1):
         'list': mailing_list,
         'held_messages': held_messages,
         'form': form,
+        'ACTION_CHOICES': ACTION_CHOICES,
         }
     return render(request, 'postorius/lists/held_messages.html', context)
 
 
+@require_http_methods(['POST'])
 @login_required
 @list_moderator_required
 def moderate_held_message(request, list_id):
-    if request.method != 'POST':
-        return HttpResponseNotAllowed(['POST'])
-    msg_id = request.POST['msgid']
+    """Moderate one held message"""
     mailing_list = List.objects.get_or_404(fqdn_listname=list_id)
-    if 'accept' in request.POST:
-        mailing_list.accept_message(msg_id)
-        messages.success(request, _('The message was accepted'))
-    elif 'reject' in request.POST:
-        mailing_list.reject_message(msg_id)
-        messages.success(request, _('The message was rejected'))
-    elif 'discard' in request.POST:
-        mailing_list.discard_message(msg_id)
-        messages.success(request, _('The message was discarded'))
+    msg = mailing_list.get_held_message(request.POST['msgid'])
+    moderation_choice = request.POST.get('moderation_choice')
+    reason = request.POST.get('reason')
+
+    try:
+        if 'accept' in request.POST:
+            mailing_list.accept_message(msg.request_id)
+            messages.success(request, _('The message was accepted'))
+        elif 'reject' in request.POST:
+            mailing_list.reject_message(msg.request_id, reason=reason)
+            messages.success(request, _('The message was rejected'))
+        elif 'discard' in request.POST:
+            mailing_list.discard_message(msg.request_id)
+            messages.success(request, _('The message was discarded'))
+    except HTTPError as e:
+        if e.code == 404:
+            messages.error(
+                request,
+                _('Held message was not found.'))
+            return redirect('list_held_messages', list_id)
+        else:
+            raise
+
+    moderation_choices = dict(ACTION_CHOICES)
+    if moderation_choice in moderation_choices:
+        member = get_member_or_nonmember(mailing_list, msg.sender)
+        if member is not None:
+            member.moderation_action = moderation_choice
+            member.save()
+            messages.success(
+                request,
+                _('Moderation action for {} set to {}'.format(
+                    member, moderation_choices[moderation_choice])))
+        else:
+            messages.error(
+                request,
+                _('Failed to set moderation action for {}'.format(msg.sender)))
     return redirect('list_held_messages', list_id)
 
 
@@ -564,15 +701,13 @@ def _get_choosable_styles(request):
     styles = Style.objects.all()
     options = [(style['name'], style['description'])
                for style in styles['styles']]
-    # Reorder to put the default at the beginning
-    for style_option in options:
-        if style_option[0] == styles['default']:
-            options.remove(style_option)
-            options.insert(0, style_option)
-            break
     return options
 
 
+def _get_default_style():
+    return Style.objects.all()['default']
+
+
 @login_required
 @superuser_required
 def list_new(request, template='postorius/lists/new.html'):
@@ -608,6 +743,8 @@ def list_new(request, template='postorius/lists/new.html'):
                 list_settings["advertised"] = form.cleaned_data['advertised']
                 list_settings.save()
                 messages.success(request, _("List created"))
+                mailinglist_created.send(sender=List,
+                                         list_id=mailing_list.list_id)
                 return redirect("list_summary",
                                 list_id=mailing_list.list_id)
             # TODO catch correct Error class:
@@ -615,19 +752,20 @@ def list_new(request, template='postorius/lists/new.html'):
                 # Right now, there is no good way to detect that this is a
                 # duplicate mailing list request other than checking the
                 # reason for 400 error.
-                if e.reason == b'Mailing list exists':
+                if e.reason == 'Mailing list exists':
                     form.add_error(
                         'listname', _('Mailing List already exists.'))
                     return render(request, template, {'form': form})
                 # Otherwise just render the generic error page.
                 return render(request, 'postorius/errors/generic.html',
-                              {'error': e.msg.decode()})
+                              {'error': e.msg})
         else:
             messages.error(request, _('Please check the errors below'))
     else:
         form = ListNew(choosable_domains, choosable_styles, initial={
             'list_owner': request.user.email,
             'advertised': True,
+            'list_style': _get_default_style(),
             })
     return render(request, template, {'form': form})
 
@@ -637,6 +775,30 @@ def _unique_lists(lists):
     return {mlist.list_id: mlist for mlist in lists}.values()
 
 
+def _get_mail_host(web_host):
+    """Get the mail_host for a web_host if FILTER_VHOST is true and there's
+    only one mail_host for this web_host.
+    """
+    if not getattr(settings, 'FILTER_VHOST', False):
+        return None
+    mail_hosts = []
+    use_web_host = False
+    for domain in Domain.objects.all():
+        try:
+            if (MailDomain.objects.get(
+                    mail_domain=domain.mail_host).site.domain == web_host):
+                if domain.mail_host not in mail_hosts:
+                    mail_hosts.append(domain.mail_host)
+        except MailDomain.DoesNotExist:
+            use_web_host = True
+    if len(mail_hosts) == 1:
+        return mail_hosts[0]
+    elif len(mail_hosts) == 0 and use_web_host:
+        return web_host
+    else:
+        return None
+
+
 @login_required
 def list_index_authenticated(request):
     """Index page for authenticated users.
@@ -659,9 +821,14 @@ def list_index_authenticated(request):
 
     # Get all the mailing lists for the current user.
     all_lists = []
+    mail_host = _get_mail_host(request.get_host().split(':')[0])
     for user_email in user_emails:
         try:
-            all_lists.extend(client.find_lists(user_email, role=role))
+            all_lists.extend(
+                client.find_lists(user_email,
+                                  role=role,
+                                  mail_host=mail_host,
+                                  count=sys.maxsize))
         except HTTPError:
             # No lists exist with the given role for the given user.
             pass
@@ -669,11 +836,14 @@ def list_index_authenticated(request):
     # just redirect them to the index page with all lists.
     if len(all_lists) == 0 and role is None:
         return redirect(reverse('list_index') + '?all-lists')
-    # Render the list index page.
+    # Render the list index page with `check_advertised = False` since we don't
+    # need to check for advertised list given that all the users are related
+    # and know about the existence of the list anyway.
     context = {
         'lists': _unique_lists(all_lists),
         'domain_count': len(choosable_domains),
-        'role': role
+        'role': role,
+        'check_advertised': False,
     }
     return render(
         request,
@@ -696,8 +866,9 @@ def list_index(request, template='postorius/index.html'):
     def _get_list_page(count, page):
         client = get_mailman_client()
         advertised = not request.user.is_superuser
+        mail_host = _get_mail_host(request.get_host().split(":")[0])
         return client.get_list_page(
-            advertised=advertised, count=count, page=page)
+            advertised=advertised, mail_host=mail_host, count=count, page=page)
 
     lists = paginate(
         _get_list_page, request.GET.get('page'), request.GET.get('count'),
@@ -707,6 +878,7 @@ def list_index(request, template='postorius/index.html'):
 
     return render(request, template,
                   {'lists': lists,
+                   'check_advertised': True,
                    'all_lists': True,
                    'domain_count': len(choosable_domains)})
 
@@ -719,6 +891,7 @@ def list_delete(request, list_id):
     the_list = List.objects.get_or_404(fqdn_listname=list_id)
     if request.method == 'POST':
         the_list.delete()
+        mailinglist_deleted.send(sender=List, list_id=list_id)
         return redirect("list_index")
     else:
         submit_url = reverse('list_delete',
@@ -731,21 +904,44 @@ def list_delete(request, list_id):
 
 @login_required
 @list_moderator_required
-def list_subscription_requests(request, list_id):
+def list_pending_confirmations(request, list_id):
     """Shows a list of subscription requests.
     """
+    return _list_subscriptions(
+        request=request,
+        list_id=list_id,
+        token_owner=TokenOwner.subscriber,
+        template='postorius/lists/pending_confirmations.html',
+        page_title=_('Subscriptions pending user confirmation'),
+        )
+
+
+@login_required
+@list_moderator_required
+def list_subscription_requests(request, list_id):
+    """Shows a list of subscription requests."""
+    return _list_subscriptions(
+        request=request,
+        list_id=list_id,
+        token_owner=TokenOwner.moderator,
+        template='postorius/lists/subscription_requests.html',
+        page_title=_('Subscriptions pending approval'),
+        )
+
+
+def _list_subscriptions(request, list_id, token_owner, template, page_title):
     m_list = List.objects.get_or_404(fqdn_listname=list_id)
     requests = [req
-                for req in m_list.requests
-                if req['token_owner'] == 'moderator']
+                for req in m_list.get_requests(token_owner=token_owner)]
     paginated_requests = paginate(
         requests,
         request.GET.get('page'),
         request.GET.get('count', 25))
     page_subtitle = '(%d)' % len(requests)
-    return render(request, 'postorius/lists/subscription_requests.html',
+    return render(request, template,
                   {'list': m_list,
                    'paginated_requests': paginated_requests,
+                   'page_title': page_title,
                    'page_subtitle': page_subtitle})
 
 
@@ -790,7 +986,8 @@ SETTINGS_SECTION_NAMES = (
     ('digest', _('Digest')),
     ('message_acceptance', _('Message Acceptance')),
     ('archiving', _('Archiving')),
-    ('subscription_policy', _('Subscription Policy')),
+    ('subscription_policy', _('Member Policy')),
+    ('bounce_processing', _('Bounce Processing')),
 )
 
 SETTINGS_FORMS = {
@@ -801,7 +998,8 @@ SETTINGS_FORMS = {
     'digest': DigestSettingsForm,
     'message_acceptance': MessageAcceptanceForm,
     'archiving': ArchiveSettingsForm,
-    'subscription_policy': ListSubscriptionPolicyForm,
+    'subscription_policy': MemberPolicyForm,
+    'bounce_processing': BounceProcessingForm,
 }
 
 
@@ -837,13 +1035,27 @@ def list_settings(request, list_id=None, visible_section=None,
                     if key in form_class.mlist_properties:
                         setattr(m_list, key, form.cleaned_data[key])
                     else:
-                        list_settings[key] = form.cleaned_data[key]
+                        val = form.cleaned_data[key]
+                        # Empty list isn't really a valid value and Core
+                        # interprets empty string as empty value for
+                        # ListOfStringsField. We are doing it here instead of
+                        # ListOfStringsField.to_python() because output from
+                        # list of strings is expected to be a list and the NULL
+                        # value is hence an empty list. The serialization of
+                        # the empty list is for us empty string, hence we are
+                        # doing this here. Technically, it can be done outside
+                        # of this view, but there aren't any other use cases
+                        # where we'd want an empty list of strings.
+                        if val == []:
+                            val = ''
+                        list_settings[key] = val
                 list_settings.save()
                 messages.success(request, _('The settings have been updated.'))
+                mailinglist_modified.send(sender=List, list_id=m_list.list_id)
             except HTTPError as e:
                 messages.error(
                     request,
-                    _('An error occured: ') + e.reason.decode('utf-8'))
+                    _('An error occurred: ') + e.reason)
             return redirect('list_settings', m_list.list_id, visible_section)
     else:
         form = form_class(initial=initial_data, mlist=m_list)
@@ -888,7 +1100,7 @@ def remove_role(request, list_id=None, role=None, address=None,
             the_list.remove_role(role, address)
         except HTTPError as e:
             messages.error(request, _('The user could not be removed: %(msg)s')
-                           % {'msg': e.msg.decode()})
+                           % {'msg': e.msg})
             return redirect('list_members', the_list.list_id, role)
         messages.success(request, _('The user %(address)s has been removed'
                                     ' from the %(role)s group.')
@@ -931,49 +1143,8 @@ def remove_all_subscribers(request, list_id):
 @login_required
 @list_owner_required
 def list_bans(request, list_id):
-    """
-    Ban or unban email addresses.
-    """
-    # Get the list and cache the archivers property.
-    m_list = List.objects.get_or_404(fqdn_listname=list_id)
-    ban_list = m_list.bans
-
-    # Process form submission.
-    if request.method == 'POST':
-        if 'add' in request.POST:
-            addban_form = ListAddBanForm(request.POST)
-            if addban_form.is_valid():
-                try:
-                    ban_list.add(addban_form.cleaned_data['email'])
-                    messages.success(request, _(
-                        'The email {} has been banned.'.format(
-                            addban_form.cleaned_data['email'])))
-                except HTTPError as e:
-                    messages.error(
-                        request, _('An error occured: %s') % e.reason)
-                except ValueError as e:
-                    messages.error(request, _('Invalid data: %s') % e)
-                return redirect('list_bans', list_id)
-        elif 'del' in request.POST:
-            try:
-                ban_list.remove(request.POST['email'])
-                messages.success(request, _(
-                    'The email {} has been un-banned'.format(
-                        request.POST['email'])))
-            except HTTPError as e:
-                messages.error(request, _('An error occured: %s') % e.reason)
-            except ValueError as e:
-                messages.error(request, _('Invalid data: %s') % e)
-            return redirect('list_bans', list_id)
-    else:
-        addban_form = ListAddBanForm()
-    banned_addresses = paginate(
-        list(ban_list), request.GET.get('page'), request.GET.get('count'))
-    return render(request, 'postorius/lists/bans.html',
-                  {'list': m_list,
-                   'addban_form': addban_form,
-                   'banned_addresses': banned_addresses,
-                   })
+    return bans_view(
+        request, list_id=list_id, template='postorius/lists/bans.html')
 
 
 @login_required
@@ -1022,7 +1193,7 @@ def list_header_matches(request, list_id):
                     errors.append(e)
             for e in errors:
                 messages.error(
-                    request, _('An error occured: %s') % e.reason)
+                    request, _('An error occurred: %s') % e.reason)
             if not errors:
                 messages.success(request, _('The header matches were'
                                             ' successfully modified.'))
@@ -1040,3 +1211,67 @@ def list_header_matches(request, list_id):
         'list': m_list,
         'formset': formset,
         })
+
+
+def confirm_token(request, list_id):
+    """Confirm token sent via Mailman Core.
+
+    This view allows confirming tokens sent via Mailman Core for confirming
+    subscriptions or verifying email addresses. This is an un-authenticated
+    view and the authentication is done by assuming the secrecy of the token
+    sent.
+
+    :param request: The Django request object.
+    :param list_id: The current mailinglist id.
+
+    """
+    m_list = List.objects.get_or_404(fqdn_listname=list_id)
+    if request.method == 'POST':
+        form = TokenConfirmForm(request.POST)
+        # If the token is None or something, just raise 400 error.
+        if not form.is_valid():
+            raise HTTPError(request.path, 400, _('Invalid confirmation token'),
+                            None, None)
+        token = form.cleaned_data.get('token')
+        try:
+            pending_req = m_list.get_request(token)
+        except HTTPError as e:
+            if e.code == 404:
+                raise HTTPError(request.path, 404,
+                                _('Token expired or invalid.'), None, None)
+            raise
+        # Since we only accept the token there isn't any need for Form data. We
+        # just need a POST request at this URL to accept the token.
+        m_list.moderate_request(token, action='accept')
+        return redirect('list_summary', m_list.list_id)
+    # Get the token from url parameter.
+    token = request.GET.get('token')
+    try:
+        pending_req = m_list.get_request(token)
+    except HTTPError as e:
+        if e.code == 404:
+            raise HTTPError(request.path, 404, _('Token expired or invalid.'),
+                            None, None)
+        raise
+
+    # If this is a token pending moderator approval, they need to login and
+    # approve it from the pending requests page.
+    if pending_req.get('token_owner') == 'moderator':
+        return redirect('list_subscription_requests', m_list.list_id)
+
+    token_type = pending_req.get('type')
+    form = TokenConfirmForm(initial=dict(token=token))
+    # Show the display_name if it is not None or "".
+    if pending_req.get('display_name'):
+        addr = email.utils.formataddr((pending_req.get('display_name'),
+                                       pending_req.get('email')))
+    else:
+        addr = pending_req.get('email')
+
+    return render(request, 'postorius/lists/confirm_token.html', {
+        'mlist': m_list,
+        'addr': addr,
+        'token': token,
+        'type': token_type,
+        'form': form,
+    })
