@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2018-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -17,18 +17,29 @@
 # Postorius.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from django.conf import settings
 from django.core.exceptions import MultipleObjectsReturned
 from django.db import IntegrityError
-from django.conf import settings
-from django.http import Http404, HttpResponseBadRequest, HttpResponse
+from django.http import Http404, HttpResponse, HttpResponseBadRequest
 from django.shortcuts import redirect
-from django.urls import reverse_lazy
+from django.urls import reverse
 from django.views.decorators.http import require_safe
-from django.views.generic import (
-    CreateView, UpdateView, ListView, DeleteView)
+from django.views.generic import CreateView, DeleteView, ListView, UpdateView
 
-from postorius.models import Domain, EmailTemplate, List
 from postorius.auth.mixins import DomainOwnerMixin, ListOwnerMixin
+from postorius.forms.list_forms import TemplateUpdateForm
+from postorius.models import Domain, EmailTemplate, List
+
+
+def _clean_with_no_strip(field, data):
+    """Clean Django's CharField field without strip=True.
+
+    :param field: Instance of the Charfield.
+    :param data: The un-cleaned data from the form.
+    :returns: Cleaned data.
+    """
+    field.strip = False
+    return field.clean(data)
 
 
 class ListContextMixin:
@@ -57,11 +68,13 @@ class ListTemplateCreateView(ListOwnerMixin, ListContextMixin, CreateView):
     fields = ['name', 'data']
 
     def get_success_url(self):
-        return reverse_lazy(
+        return reverse(
             'list_template_list', args=(self.kwargs['list_id'],))
 
     def form_valid(self, form):
         formdata = form.cleaned_data
+        formdata['data'] = _clean_with_no_strip(form.fields['data'],
+                                                form.data['data'])
         formdata['identifier'] = self.kwargs['list_id']
         formdata['context'] = 'list'
         email_template = EmailTemplate(**formdata)
@@ -83,10 +96,10 @@ class ListTemplateUpdateView(ListOwnerMixin, ListContextMixin, UpdateView):
 
     template_name = 'postorius/lists/template_update.html'
     model = EmailTemplate
-    fields = ['data']
+    form_class = TemplateUpdateForm
 
     def get_success_url(self):
-        return reverse_lazy(
+        return reverse(
             'list_template_list', args=(self.kwargs['list_id'],))
 
 
@@ -96,7 +109,7 @@ class ListTemplateDeleteView(ListOwnerMixin, ListContextMixin, DeleteView):
     model = EmailTemplate
 
     def get_success_url(self):
-        return reverse_lazy(
+        return reverse(
             'list_template_list', args=(self.kwargs['list_id'],))
 
 
@@ -128,11 +141,13 @@ class DomainTemplateCreateView(
     header = 'New Template'
 
     def get_success_url(self):
-        return reverse_lazy('domain_template_list',
-                            args=(self.kwargs['domain'],))
+        return reverse('domain_template_list',
+                       args=(self.kwargs['domain'],))
 
     def form_valid(self, form):
         formdata = form.cleaned_data
+        formdata['data'] = _clean_with_no_strip(form.fields['data'],
+                                                form.data['data'])
         formdata['identifier'] = self.kwargs['domain']
         formdata['context'] = 'domain'
         template = EmailTemplate(**formdata)
@@ -155,12 +170,12 @@ class DomainTemplateUpdateView(
 
     model = EmailTemplate
     template_name = 'postorius/domain/template_add.html'
-    fields = ['data']
+    form_class = TemplateUpdateForm
     header = 'Edit Template'
 
     def get_success_url(self):
-        return reverse_lazy('domain_template_list',
-                            args=(self.kwargs['domain'],))
+        return reverse('domain_template_list',
+                       args=(self.kwargs['domain'],))
 
 
 class DomainTemplateDeleteView(
@@ -170,8 +185,8 @@ class DomainTemplateDeleteView(
     model = EmailTemplate
 
     def get_success_url(self):
-        return reverse_lazy('domain_template_list',
-                            args=(self.kwargs['domain'],))
+        return reverse('domain_template_list',
+                       args=(self.kwargs['domain'],))
 
 
 @require_safe
